@@ -35,22 +35,25 @@ be provided at some point... watch this space!
 
 TODOs:
 
-  For 2.1.0:
+  For 2.2.0:
+   * Add Sphinx docs.
    * Preserve comments from read -> write (needs full-line/inline comment
-     separation?). Can use separate comment dicts in Block and Decay, and
+     separation). Can use separate comment dicts in Block and Decay, and
      attach a multiline .comment attr to the returned/written dicts.
-   * Only allow integers to be used as block entry indices, to allow multiple
-     values (request, if possible, from Anders & Are).
 
   Later/maybe:
    * Identify HERWIG decay matrix element to use in ISAWIG
    * Handle RPV SUSY in ISAWIG
 """
 
-__author__ = "Andy Buckley <andy.buckley@cern.ch"
-__version__ = "2.0.0"
+__author__ = "Andy Buckley <andy.buckley@cern.ch>"
+__version__ = "2.1.0"
 
 
+
+###############################################################################
+## Private utility functions
+
 def _mkdict():
     """Try to return an empty ordered dict, but fall back to normal dict if necessary"""
     try:
@@ -81,6 +84,16 @@ def _autostr(var, precision=8):
         return ("% ." + str(precision) + "e") % var
     return str(var)
 
+def _autotuple(a):
+    """Automatically convert the supplied iterable to a scalar or tuple as appropriate."""
+    if len(a) == 1:
+        return a[0]
+    return tuple(a)
+
+
+
+###############################################################################
+## Exceptions
 
 class AccessError(Exception):
     "Exception object to be raised when a SLHA block is accessed in an invalid way"
@@ -98,6 +111,9 @@ class ParseError(Exception):
 
 
 
+###############################################################################
+## The data block, decay and particle classes
+
 class Block(object):
     """
     Object representation of any BLOCK elements read from an SLHA file.
@@ -113,10 +129,11 @@ class Block(object):
     methods for iteration. Purely iterating over the object behaves like keys(),
     as for an ordinary dict.
 
-    Multiple indices are possible, especially for entries in mixing matrix
-    blocks. These are now implemented in the natural way, e.g. for access to the
-    (1,2) element of a mxing matrix block, use bmix[1,2] = 0.123 and print
-    bmix[1,2]. The value() and set_value() functions behave similarly.
+    Multiple (integer) indices are possible, especially for entries in mixing
+    matrix blocks. These are now implemented in the natural way, e.g. for access
+    to the (1,2) element of a mixing matrix block, use bmix[1,2] = 0.123 and
+    print bmix[1,2]. The value() and set_value() functions behave
+    similarly. Multi-element values are also permitted.
 
     It is possible, although not usual, to store unindexed values in a
     block. This is only supported when that entry is the only one in the block,
@@ -133,29 +150,26 @@ class Block(object):
             self.entries.update(entries)
         self.q = _autotype(q)
 
-    def add_entry(self, entry):
-        """Add an entry to the block from an iterable (i.e. list or tuple) or
-        whitespace-separated string.
+    # TODO: Rename? To what?
+    def add_entry(self, args):
+        """Add an entry to the block from an iterable (i.e. list or tuple) of
+        strings, or from a whitespace-separated string.
 
-        Indexing will be determined automatically such that there is always a
-        single-element value: multi-value or None indices may be constructed
-        implicitly.
+        This method is just for convenience: it splits the single string
+        argument if necessary and converts the list of strings into numeric
+        types when possible. For the treatment of the resulting iterable see the
+        set_value method.
         """
         ## If the argument is a single string, split it and proceed
-        if type(entry) is str:
-            entry = entry.split()
+        if type(args) is str:
+            args = args.split()
         ## Check that the arg is an iterable
-        if not hasattr(entry, "__iter__"):
+        if not hasattr(args, "__iter__"):
             raise AccessError("Block entries must be iterable")
         ## Auto-convert the types in the list
-        entry = map(_autotype, entry)
+        args = map(_autotype, args)
         ## Add the entry to the map, with appropriate indices
-        if len(entry) == 1:
-            self.entries[None] = entry[0]
-        elif len(entry) == 2:
-            self.entries[entry[0]] = entry[1]
-        else:
-            self.entries[tuple(entry[:-1])] = entry[-1]
+        self.set_value(*args)
 
     def is_single_valued(self):
         """Return true if there is only one entry, and it has no index: the
@@ -173,11 +187,17 @@ class Block(object):
         return self.entries[key]
 
     def set_value(self, *args):
-        """Set a value in the block with supplied key, val args (or just val for
-        an unindexed block).
+        """Set a value in the block via supplied key/value arguments.
 
-        If no key is given, then the block must contain only one non-indexed
-        value otherwise an AccessError exception will be raised.\
+        Indexing is determined automatically: any leading integers will be
+        treated as a multi-dimensional index, with the remaining entries being a
+        (potentially multi-dimensional) value. If all N args are ints, then the
+        first N-1 are treated as the index and the Nth as the value.
+
+        If there is only one arg it will be treated as the value of a
+        single-valued block. In this case the block must already contain at most
+        one non-indexed value otherwise an AccessError exception will be
+        raised.\
         """
         if len(args) == 0:
             raise AccessError("set_value() called without arguments")
@@ -186,7 +206,16 @@ class Block(object):
                 raise AccessError("Tried to set a unique value on a multi-value block")
             self.entries[None] = args[0]
         else:
-            self.entries[tuple(args[:-1])] = args[-1]
+            ## Find the first non-integer -- all previous items are indices
+            i_first_nonint = -1
+            for i, x in enumerate(args):
+                if type(x) is not int:
+                    i_first_nonint = i
+                    break
+            if i_first_nonint == 0:
+                raise AccessError("Attempted to set a block entry with a non-integer(s) index")
+            else:
+                self.entries[_autotuple(args[:i_first_nonint])] = _autotuple(args[i_first_nonint:])
 
     def has_key(self, key):
         """Does the block have the given key?"""
@@ -214,6 +243,8 @@ class Block(object):
         return self.entries[key]
 
     def __setitem__(self, key, value):
+        if key is not None and type(key) is not int and not all(type(x) is int for x in key):
+            raise AccessError("Attempted to set a block entry with a non-integer(s) index")
         self.entries[key] = value
 
     def __cmp__(self, other):
@@ -228,7 +259,7 @@ class Block(object):
                 s = "%r" % k
             else:
                 s = ",".join("%r" % subindex for subindex in k)
-            s += " : %r" % v
+            s += " : %s" % repr(v)
             return s
         s += " { " + "; ".join(_format_kv(k, v) for k, v in self.items()) + " }"
         return s
@@ -299,6 +330,10 @@ class Particle(object):
         return s
 
 
+
+###############################################################################
+## SLHA parsing and writing functions
+
 def readSLHA(spcstr, ignorenobr=False):
     """
     Read an SLHA definition from a string, returning dictionaries of blocks and
@@ -626,7 +661,6 @@ def pdgid2herwigid(pdgid):
 ###############################################################################
 ## ISAWIG format reading/writing
 
-
 def readISAWIG(isastr, ignorenobr=False):
     """
     Read a spectrum definition from a string in the ISAWIG format, returning
@@ -700,7 +734,7 @@ def readISAWIG(isastr, ignorenobr=False):
     blocks["MINPAR"] = Block("MINPAR")
     blocks["MINPAR"][3] = TANB
     blocks["ALPHA"] = Block("ALPHA")
-    blocks["ALPHA"][None] = ALPHAH
+    blocks["ALPHA"].set_value(ALPHAH)
     #
     ## Neutralino mixing matrix
     blocks["NMIX"] = Block("NMIX")
@@ -1014,9 +1048,9 @@ def writeISAWIG(blocks, decays, ignorenobr=False, precision=8):
     return out
 
 
-###############################################################################
-## File-level functions
 
+###############################################################################
+## File-level read/write functions
 
 def readSLHAFile(spcfilename, **kwargs):
     """
@@ -1082,7 +1116,6 @@ def writeISAJETFile(isafilename, blocks, decays, **kwargs):
 ###############################################################################
 ## Main function for module testing
 
-
 if __name__ == "__main__":
     import sys
     for a in sys.argv[1:]:
